/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is
 *  William Liao <william.firefox@m2k.com.tw>.
 * Portions created by the Initial Developer are Copyright (C) 2004
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
var gM2kNotifier = null;

function m2kLoad()
{
  gM2kNotifier = new mail2knotifier();
  gM2kNotifier.init();
}

function m2kUnload()
{
  gM2kNotifier.destroy();
  gM2kNotifier = null;
}

///////////////////////
// mail2knotifier Object
function mail2knotifier()
{
  return this;
}

mail2knotifier.prototype =
{
	gXMLHttpRequest: null,
	oldNewMsgCount: null,
	mDomain: "mail2knotifier.",
	mBranch: null,
	mProfileIsLoaded: null,
	PASSWD: null,
	LOGGED_IN: null,
	bTimerID: null,
	bLoginWindowID: null,
	
	init: function()
	{
		this.oldNewMsgCount = 0;
		this.mProfileIsLoaded = false;
		this.PASSWD = "";
		this.LOGGED_IN = false;
		this.bTimerID = null;
		this.bLoginWindowID = null;
		//setup preference service
		var prefService = "@mozilla.org/preferences-service;1";
		prefService = Components.classes[prefService].getService(Components.interfaces.nsIPrefService);
		this.mBranch = prefService.getBranch(this.mDomain);
		
		//add preference observer
		var pbi = this.mBranch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
		pbi.addObserver("", this, false);
		
		this.initPref();
		this.mProfileIsLoaded = true;
		if (this.LOGGED_IN)
		{
			this.setupShortInterval(1000);
		}

	},
	
	destroy: function()
	{
		try
		{
			//remove preference observer
			var pbi = this.mBranch.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
			pbi.removeObserver("", this);
		}
		catch(e) {}
		
		//Destroy variables at the end
		this.gXMLHttpRequest = null;
		this.oldNewMsgCount = null;
		this.mDomain = null;
		this.mBranch = null;
		this.mProfileIsLoaded = null;
		this.PASSWD = null;
		this.LOGGED_IN = null;
		this.bTimerID = null;
		this.bLoginWindowID = null;
	},
		
	observe: function (aSubject, aTopic, aData)
	{
		switch (aData)
		{
			//When options changed, reset variables and checkMail
			case "pref.remember":
				if (this.mBranch.getBoolPref ("pref.remember"))
				{
					this.passManager ("add");
				}
				else
				{
					this.passManager ("remove");
				}
				break;
			
			case "pref.server":
			case "pref.userid":
				if (this.mProfileIsLoaded)
				{
					this.resetMsgStatus();
					this.checkMail();
				}
				break;
				
			case "pref.freq":
				if (this.mProfileIsLoaded)
				{
					this.setupInterval();
				}

			case "lastcheck.time":
			case "lastcheck.newmsg":
			case "prefs.customized-server":
				break;

			default	:
				//if (this.mProfileIsLoaded)
				//{
				//	alert ("What's wrong with this?");
				//}
				break;
		}
	},
		
	initPref: function ()
	{
		var test = null;
		//Initialized variables if has not initialized
		try 
		{
			test = this.mBranch.getCharPref ("pref.server");
		} 
		catch (e)
		{
			this.mBranch.setCharPref ("pref.server", "mail2000.com.tw");
		}

		try 
		{
			test = this.mBranch.getCharPref ("pref.userid");
		} 
		catch (e)
		{
			this.mBranch.setCharPref ("pref.userid", "");
		}

		try 
		{
			test = this.mBranch.getIntPref ("pref.freq");
		} 
		catch (e)
		{
			this.mBranch.setIntPref ("pref.freq", 5);
		}
		
		try 
		{
			test = this.mBranch.getBoolPref ("pref.remember");
		} 
		catch (e)
		{
			this.mBranch.setBoolPref ("pref.remember", false);
		}
		
		try 
		{
			test = this.mBranch.getCharPref ("lastcheck.time");
		} 
		catch (e)
		{
			this.mBranch.setCharPref ("lastcheck.time", "0");
		}
		
		try 
		{
			test = this.mBranch.getIntPref ("lastcheck.newmsg");
		} 
		catch (e)
		{
			this.mBranch.setIntPref ("lastcheck.newmsg", 0);
		}

		try 
		{
			test = this.mBranch.getBoolPref ("pref.customized-server");
		} 
		catch (e)
		{
			this.mBranch.setBoolPref ("pref.customized-server", false);
		}
		
		this.passManager("getpass");
		
		if (this.mBranch.getBoolPref ("pref.remember"))
		{
			this.LOGGED_IN = true;
		}
		else
		{
			this.LOGGED_IN = false;
		}
	},
	
	resetMsgStatus: function()
	{
		var icon = document.getElementById("mail2knotifier-statusbar");
		this.oldNewMsgCount = 0;
		icon.removeAttribute("label");
		icon.setAttribute("src","chrome://mail2knotifier/skin/small/no_new_mail.png");
		this.mBranch.setCharPref("lastcheck.time", "0");
		this.mBranch.setIntPref("lastcheck.newmsg", 0);
	},
	
	passManager: function(aAction)
	{

		var host = "chrome://mail2knotifier/"+this.mBranch.getCharPref("pref.server")+"";

		//var loginInfo = new nsLoginInfo(hostname, formSubmitURL, httprealm, username, password, 
		//                                     usernameField, passwordField);

		switch (aAction)
		{
			case "add":
				if ("@mozilla.org/passwordmanager;1" in Components.classes) {
					var passManager = Components.classes["@mozilla.org/passwordmanager;1"].createInstance();
					passManager = passManager.QueryInterface(Components.interfaces.nsIPasswordManager);
					try
					{
						passManager.removeUser(host, gM2kNotifier.mBranch.getCharPref("pref.userid"));
					} 
					catch (e) { }
					passManager.addUser(host, gM2kNotifier.mBranch.getCharPref("pref.userid"), this.PASSWD);
				}
				else if ("@mozilla.org/login-manager;1" in Components.classes) {
					
					var passManager = Components.classes["@mozilla.org/login-manager;1"]
										.getService(Components.interfaces.nsILoginManager);
					var nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",
			                                     Components.interfaces.nsILoginInfo, "init");				
					var gM2kLoginInfo = new nsLoginInfo(host, 'M2kLogin', null, 
											gM2kNotifier.mBranch.getCharPref("pref.userid"), this.PASSWD, "", "");				
					try
					{
						passManager.removeLogin(gM2kLoginInfo);
					} 
					catch (e) { }
					passManager.addLogin(gM2kLoginInfo);
				}
				break;
			case "remove":
				if ("@mozilla.org/passwordmanager;1" in Components.classes) {
					var passManager = Components.classes["@mozilla.org/passwordmanager;1"].createInstance();
					passManager = passManager.QueryInterface(Components.interfaces.nsIPasswordManager);
					try
					{
						passManager.removeUser(host, gM2kNotifier.mBranch.getCharPref("pref.userid"));
					} 
					catch (e) { }				}
				else if ("@mozilla.org/login-manager;1" in Components.classes) {
					var passManager = Components.classes["@mozilla.org/login-manager;1"]
										.getService(Components.interfaces.nsILoginManager);
					var nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",
			                                     Components.interfaces.nsILoginInfo, "init");
					// Find users for this extension 
					var logins = passwordManager.findLogins({}, host, 'M2kLogin', null);
					try
					{
						for (var i = 0; i < logins.length; i++) {
						  if (logins[i].username == gM2kNotifier.mBranch.getCharPref("pref.userid")) {
						     passwordManager.removeLogin(logins[i]);
						     //break;
						  }
						}
					} 
					catch (e) { }
				}
				
				break;
			case "getpass":
				if ("@mozilla.org/passwordmanager;1" in Components.classes) {
					var passManager = Components.classes["@mozilla.org/passwordmanager;1"].createInstance(Components.interfaces.nsIPasswordManagerInternal);
					var gethost = {value:""};
					var getuser =  {value:""};
					var getpasswd = {value:""};
					try
					{
						passManager.findPasswordEntry(host, gM2kNotifier.mBranch.getCharPref("pref.userid"), null, gethost, getuser, getpasswd);
					}
					catch(e) { }
					if (getpasswd.value == null)
					{
						this.PASSWD = "";
						this.LOGGED_IN = false;		//No empty password!
					}
					else
					{
						this.PASSWD = getpasswd.value;
					}
				}
				else if ("@mozilla.org/login-manager;1" in Components.classes) {

					var passManager = Components.classes["@mozilla.org/login-manager;1"]
										.getService(Components.interfaces.nsILoginManager);
					var nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",
			                                     Components.interfaces.nsILoginInfo, "init");
					var getpasswd = {value:""};
					try
					{
						// Find users for the given parameters
						var logins = passManager.findLogins({}, host, 'M2kLogin', null);
						  
						// Find user from returned array of nsILoginInfo objects
						for (var i = 0; i < logins.length; i++) {
						  if (logins[i].username == gM2kNotifier.mBranch.getCharPref("pref.userid")) {
						     getpasswd = logins[i].password;
						     break;
						  }
						}
					}
					catch(e) { }
					if (getpasswd == "")
					{
						this.PASSWD = "";
						this.LOGGED_IN = false;		//No empty password!
					}
					else
					{
						this.PASSWD = getpasswd;
					}
				}
				
				break;
			default:
				//alert ("What's going on here?");
				break;
		}
	},
	
	
	setupShortInterval: function (aNum)
	{
		if (this.bTimerID)
		{
			window.clearInterval (gM2kNotifier.bTimerID);
		}

		if (this.LOGGED_IN)
		{
			this.bTimerID = window.setInterval ("gM2kNotifier.intervalListener()", aNum);
		}
	},
	
	setupInterval: function ()
	{
		if (this.bTimerID)
		{
			window.clearInterval (gM2kNotifier.bTimerID);
		}

		if ((this.LOGGED_IN && (this.mBranch.getIntPref("pref.freq") > 0)))
		{
			this.bTimerID = window.setInterval ("gM2kNotifier.intervalListener()", (gM2kNotifier.mBranch.getIntPref("pref.freq")*60*1000));
		}
	},
	
	intervalListener: function ()
	{
		this.checkMail();
		this.setupInterval();
	},
	
	openLoginWindow: function()
	{
		this.bLoginWindowID = window.openDialog("chrome://mail2knotifier/content/m2klogin.xul", "_blank", "chrome,resizable=no,dependent=yes");
	},
		
	openLink: function(aEvent)
	{
		if (this.LOGGED_IN)
		{
			var url = "http://"+this.mBranch.getCharPref("pref.server")+"/cgi-bin/mlogin?USERID="+encodeURI(this.mBranch.getCharPref("pref.userid"))+"&PASSWD="+encodeURI(this.PASSWD)+"";
			var myBrowser = document.getElementById("content");
			const nsIWebNavigation = Components.interfaces.nsIWebNavigation
			myBrowser.webNavigation.loadURI(url, nsIWebNavigation.LOAD_FLAGS_BYPASS_HISTORY, null, null, null);
			//An easy (read: unsecured) way to hide login information.
			//A better way is to send the data via POST method. => myBrowser.webNavigation.loadURI(url, nsIWebNavigation.LOAD_FLAGS_BYPASS_HISTORY, null, POSTDATA, HEADER);
			document.getElementById("urlbar").value = "http://www."+this.mBranch.getCharPref("pref.server")+"/";
//			openUILink(url, aEvent);
		}
		else
		{
			this.openLoginWindow();
		}
	},

	checkForMiddleClick: function (node, event)
	{
//		if (event.button == 1) {
//			eval(node.getAttribute("ondblclick"));
//			closeMenus(event.target);
//		}

		if (event.button == 1)
		{
			var thisTab = document.getElementById("content").selectedTab;
			var newTab = document.getElementById("content").addTab ("", null, null);
			document.getElementById("content").selectedTab = newTab;
			this.openLink(event);
			document.getElementById("content").selectedTab = thisTab;
		}
		
	},
		
	stopCheckMail: function ()
	{
		this.LOGGED_IN = false;
		this.setupInterval();
	},
	
	checkNow: function ()
	{
		this.LOGGED_IN = true;
		this.resetMsgStatus();
		this.setupShortInterval(1000);
	},
	
	checkMail: function()
	{
		var Now = new Date();
		if ((parseInt(gM2kNotifier.mBranch.getCharPref("lastcheck.time"))+gM2kNotifier.mBranch.getIntPref("pref.freq")*60*1000) < Now.getTime())
		{
			//Check Now
			var url = "http://"+gM2kNotifier.mBranch.getCharPref("pref.server")+"/cgi-bin/chk_newmail?USERID="+gM2kNotifier.mBranch.getCharPref("pref.userid")+"&PASSWD="+gM2kNotifier.PASSWD+"";

			gXMLHttpRequest = new XMLHttpRequest();
			gXMLHttpRequest.onload = gM2kNotifier.updateMailStatus;
			gXMLHttpRequest.open("GET", url);
			gXMLHttpRequest.send(null);
			gM2kNotifier.mBranch.setCharPref("lastcheck.time", Now.getTime());
		}
		else
		{
			//Use the lastcheck.newmsg
			var icon = document.getElementById("mail2knotifier-statusbar");
			if (gM2kNotifier.mBranch.getIntPref("lastcheck.newmsg") >0 )
			{
				icon.setAttribute("label", "("+gM2kNotifier.mBranch.getIntPref("lastcheck.newmsg")+")");
				icon.setAttribute("src","chrome://mail2knotifier/skin/small/new_mail.png");
			}
			else
			{
				icon.removeAttribute("label");
				icon.setAttribute("src","chrome://mail2knotifier/skin/small/no_new_mail.png");
			}
			gM2kNotifier.oldNewMsgCount = gM2kNotifier.mBranch.getIntPref("lastcheck.newmsg");
		}
	},

	updateMailStatus: function()
	{
		var icon = document.getElementById("mail2knotifier-statusbar");
		var newmail_count;
		var x = null;
		var myResponse = gXMLHttpRequest.responseText;
		var myResponseArray = myResponse.split("\n");

		if (myResponseArray[0] == "SUCCESS")
		{
			gM2kNotifier.LOGGED_IN = true;
			newmail_count = myResponseArray.length - 2;		//The "SUCCESS" string and the trailing "\n"
			gM2kNotifier.mBranch.setIntPref("lastcheck.newmsg", newmail_count);
			for (x=1;x<=newmail_count;x++)					//Get the mail info
			{
				myResponseArray[x] = myResponseArray[x].split("\t");
			}

			if (newmail_count > 0)
			{
				icon.setAttribute("label", "("+newmail_count+")");
				icon.setAttribute("src","chrome://mail2knotifier/skin/small/new_mail.png");

				if (gM2kNotifier.oldNewMsgCount < newmail_count)
				{
					gM2kNotifier.NewMailNotification(myResponseArray);
//					this.NewMailNotificationMultiple(myResponseArray, gM2kNotifier.oldNewMsgCount);
				}
				gM2kNotifier.oldNewMsgCount = newmail_count;		// Mark the current count
			}
			else
			{
				icon.removeAttribute("label");
				icon.setAttribute("src","chrome://mail2knotifier/skin/small/no_new_mail.png");
			}

		}
		else if (myResponseArray[0] == "ERROR" || myResponseArray[0] == "ERROR5")
		{
			gM2kNotifier.LOGGED_IN = false;
			gM2kNotifier.setupInterval();
			icon.setAttribute("label", "!");
			icon.setAttribute("src","chrome://mail2knotifier/skin/small/no_new_mail.png");
			alert ("Login Error!");
			gM2kNotifier.passManager("remove");
		}
		else if (myResponseArray[0] == "NO USER")
		{
			gM2kNotifier.LOGGED_IN = false;
			gM2kNotifier.setupInterval();
			icon.setAttribute("label", "!");
			icon.setAttribute("src","chrome://mail2knotifier/skin/small/no_new_mail.png");
			alert ("Username Error!");
			gM2kNotifier.passManager("remove");
		}
		else {
			gM2kNotifier.LOGGED_IN = false;
			gM2kNotifier.setupInterval();
			icon.setAttribute("label", "!");
			icon.setAttribute("src","chrome://mail2knotifier/skin/small/no_new_mail.png");
		}

		gXMLHttpRequest = null;
		},
	
	NewMailNotification: function(aMsgInfo)		//Only display the number of new mails...
	{
		var m2kstrings	= document.getElementById("m2k_bundle");
		var msg = m2kstrings.getString("m2k.notification.msgbody.header") + " " + (aMsgInfo.length-2) + " " + m2kstrings.getString("m2k.notification.msgbody.footer");
		try
		{
			var alertService = Components.classes["@mozilla.org/alerts-service;1"]
                                         .getService(Components.interfaces.nsIAlertsService);
                                         
            //The following line will generate an error in Javascript Console.....
            alertService.showAlertNotification("chrome://mail2knotifier/skin/got_mail.png",
                                     m2kstrings.getString("m2k.notification.title"), msg, false, "", this);
        }
        catch (e) { }
	},
		
	onAlertFinished: function (aCookie)
	{
		
	},

	onAlertClickCallback: function (aCookie)
	{
//		this.openURL();
	},
	
// Function below doesn't work

	// Multiple alert is not working
/*
	NewMailNotificationMultiple: function(aMsgInfo, curMsgCount)
	{
		var x = null;
		if (curMsgCount < aMsgInfo.length-2)		// Double check
		{
			curMsgCount++;		//Set the starting position
			for (x=curMsgCount;x<=aMsgInfo.length-2;x++)
			{
				try
				{
					var msg = "Date: "+aMsgInfo[x][0]+"\nSender: "+aMsgInfo[x][2]+"\nSubject: "+aMsgInfo[x][1]+"";
					var alertService = Components.classes["@mozilla.org/alerts-service;1"]
		                                         .getService(Components.interfaces.nsIAlertsService);
		            alertService.showAlertNotification("chrome://mail2knotifier/skin/got_mail.png",
		                                     "Message "+x+" of "+(aMsgInfo.length-2)+"", msg, false, "", this);
		        }
		        catch (e) { }
		    }
		}
	},
	
	onAlertFinished: function (aCookie)
	{
		
	},
		
	openURL: function()
	{
		var url = "http://"+this.mBranch.getCharPref("pref.server")+"/cgi-bin/mlogin?USERID="+this.mBranch.getCharPref("pref.userid")+"&PASSWD="+this.PASSWD+"";
		document.getElementById("content").loadURI(url);
	},

	//Don't know how to make this working...
	onAlertClickCallback: function (aCookie)
	{
//		this.openURL();
	}
*/
}
